/*
   This program parses and evaluates strings from Language_8.
*/

public class Language_8_Examples
{
   public static void main(String[] args)
   {
      String[] programs =
      {
/*
         // With static scope, this example would be an infinite loop.
         // With dynamic scope, this example terminates.
         "( prog " +
           "( fun f1 ( lambda x ( apply f2 0 ) ) ) " +  // global f1
           "( fun f2 ( lambda y ( begin " +
                               "( fun f1 ( lambda w 10 ) ) " +  // nested f1
                               "( apply f3 0 ) ) ) ) " +
           "( fun f3 ( lambda z ( apply f1 0 ) ) ) " +  // which f1 is called?
           "( apply f1 0 ) )",
*/
         // The final result should be 100.
         // (It would be 10 using dynamic scope.)
         "( prog " +
           "( fun f1 ( lambda x 100 ) ) " +  // global f1
           "( fun f2 ( lambda y ( begin " +
                               "( fun f1 ( lambda w 10 ) ) " +  // nested f1
                               "( apply f3 0 ) ) ) ) " +
           "( fun f3 ( lambda z ( apply f1 0 ) ) ) " +  // which f1 is called?
           "( apply f2 0 ) )",

         // Demonstrate a function parameter.
         // The final result should be 321.
         // (It would be 421 using dynamic scope.)
         "( prog " +
           "( var w 300 ) " +
           "( fun f1 ( lambda x " +
                          "( begin " +
                            "( fun f2 ( lambda y ( + ( + x y ) w ) ) ) " + // nested function
                            "( apply f3 f2 ) ) ) ) " + // nested function passed as a parameter
           "( fun f3 ( lambda z " +  // z is a function parameter
                         "( begin " +
                           "( var w 400 ) " +
                           "( apply z 20 ) ) ) ) " + // call the function parameter
           "( apply f1 1 ) )",

         // Another (convoluted) example demonstrating dynamic scope.
         // (Notice that, with static scope, h is undefined in f, and x is undefined in h.)
         "( prog " +
           "( fun f ( lambda x ( apply h x ) ) ) " +  // non local reference to h
           "( fun g ( lambda y ( begin " +
                                "( fun h ( lambda z ( + z x ) ) ) " + // non local ref to x
                                "( apply f y ) ) ) ) " +
           "( apply g 5 ) )",

         // This next example shows that fun returns a function
         // value which can be directly applied to an input.
         // Notice that g does not play much of any role in this
         // example. In the next language we will use an "anonymous
         // lambda expression" for this example.
         "( var r ( apply ( fun g ( lambda x ( * x x ) ) ) 5 ) )",

         // Show that a function can be passed as a parameter.
         // The final value should be 10.
         // (With dynamic scope, the final result would be different.)
         "( prog " +
           "( var n 2 ) " +
           "( fun f ( lambda x ( * n x ) ) ) " +
           "( fun g ( lambda h ( apply h 5 ) ) ) " +
           "( begin " +
             "( var n 10 ) " +
             "( apply g f ) ) )",  // pass f as an argument

         // Show that a function can be the return value.
         // (With dynamic scope, this program would return a different result.)
         "( prog " +
           "( var x -100 ) " +
           "( fun f ( lambda x ( begin " +
                                "( fun g ( lambda y ( + y x ) ) ) " +
                                "g ) ) ) " +  // return g
           "( apply ( apply f 5 ) 100 ) )"
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         // Print the expression.
         //System.out.println( programs[i] + "\n" );

         // Build the abstract syntax tree that represents the expression.
         try
         {
            Tree ast = Parser.parse( programs[i] );

            // Print the AST as an S-expression
            //System.out.println( ast + "\n" );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the expression.
            System.out.println( AST2Infix.ast2infix( ast ) + "\n" );

            // Evaluate the expression (interpret the AST).
            try
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
               System.out.println(e);
             //e.printStackTrace();
            }


            // Create dot and png files from the AST.
            if(true)
            try
            {
               // Create the (empty) dot file.
               String baseName = String.format("Language_6.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot_ver1.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//Language_8_Examples