/*
   The function makeIncrementor defined below
   contains a nested function that is a function
   return value, and the nested function makes a
   non-local reference to a lexically scoped variable.

   Compare this example to the equivalent example
   that we wrote in Scheme.
*/

public class Language_8_Examples_3
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "( prog " +
           "( fun makeIncrementor ( lambda inc " +
                                    "( fun temp ( lambda x ( + x inc ) ) ) ) ) " +
           "( var incBy3 ( apply makeIncrementor 3 ) ) " +
           "( var incBy5 ( apply makeIncrementor 5 ) ) " +
           "( print ( apply incBy3  6 ) ) " +
           "( print ( apply incBy5  6 ) ) " +
           "( print ( apply incBy3 -2 ) ) " +
           "( print ( apply incBy5 -2 ) ) )",


         "( prog " +
           "( fun makeCounter " +
               "( lambda x " +  // return a function with no parameters
                      "( fun r ( lambda ( set x ( + x 1 ) ) ) ) ) ) " +
              "( var c1 ( apply makeCounter 1 ) ) " +
              "( var c2 ( apply makeCounter 10 ) ) " +
              "( print ( apply c1 ) ) " +  // call a function with no parameters
              "( print ( apply c1 ) ) " +
              "( print ( apply c1 ) ) " +
              "( print ( apply c2 ) ) " +
              "( print ( apply c2 ) ) " +
              "( print ( apply c2 ) ) " +
              "( print ( apply c1 ) ) )"
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         // Print the expression.
         //System.out.println( programs[i] + "\n" );

         // Build the abstract syntax tree that represents the expression.
         try
         {
            Tree ast = Parser.parse( programs[i] );

            // Print the AST as an S-expression
            //System.out.println( ast + "\n" );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the expression.
            System.out.println( AST2Infix.ast2infix( ast ) + "\n" );

            // Evaluate the expression (interpret the AST).
            try
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
               System.out.println(e);
             //e.printStackTrace();
            }


            // Create dot and png files from the AST.
            if(true)
            try
            {
               // Create the (empty) dot file.
               String baseName = String.format("Language_6.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot_ver1.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main
}//Language_8_Examples_3