/*
   This prettyPrint() method takes a Tree
   and converts it into a well formated string.

   This pretty-printer does the following.
      1) It puts the left node "inline" with the root.
      2) It handles the special case when depth=1.
      3) It handles the special case when depth=2.
      4) Closing parentheses are either aligned vertically
         or are all on the end of a line.
*/

public class PrettyPrinter
{
   public static final boolean ALIGN = false;

   public static String prettyPrint(Tree tree)
   {
      return prettyPrint(tree, "");
   }

   public static String prettyPrint(Tree tree, String indentation)
   {
      String result = "";

      if ( 0 == tree.degree() )
      {
         result += tree.getElement();
      }
      else if ( (1 == tree.depth()) || (2 == tree.depth()) )
      {
         result += "(" + tree.getElement();

         for (int i = 0; i < tree.degree(); i++)
            result += " " + prettyPrint( tree.getSubTree(i), null );

         result += ")";
      }
      else
      {
         result += "(" + tree.getElement();

         for (int i = 0; i < 2 + tree.getElement().length(); i++)
            indentation += " ";

         // recursively traverse the first branch of this tree.
         result += " " + prettyPrint( tree.getSubTree(0), indentation );
         // recursively traverse the rest of this tree.
         for (int i = 1; i < tree.degree(); i++)
            result += "\n" + indentation + prettyPrint( tree.getSubTree(i), indentation );

         result += ALIGN ? ("\n" + indentation + ")") : ")";
      }
      return result;
   }//prettyPrint()

}//PrettyPrinter