/*
  This program tests your implentations of StaticAnalysis.java.
  When your implentation is correct, the output from this
  program should look exactly like the contents of Tests_output.txt.

  You should consider creating your own test expressions to make
  sure your implementation is correct.
*/

public class Tests
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "( prog " +
           "( var x ( + x 1 ) ) " +  // can't have a recursive declaration
           "( set u ( + 0 false ) ) )",

         "( prog " +
           "( var z 10 ) " +
           "( fun f ( lambda x " +
              "( begin " +
                "( if true z ( while false ( + u v ) ) ) " +
                "( apply f x ) ) ) ) " +
           "( apply f z ) )",

         "( prog " +
           "( var x 0 ) " +
           "( var n 0 ) " +
           "( while ( < n 100000000 ) " +  // 100,000,000 iterations
             "( begin " +
               "( if ( < n 100000000 ) " +
                   "( set x n ) " +
                   "( set x u ) ) ) ) )",

         "( prog " +
           "( var i1 5 ) " +
           "( while ( > i2 0 ) " +
             "( begin " +
               "( print i1 ) " +
               "( var i3 ( - i3 1 ) ) ) ) )",

         "( prog " +
           "( var n 1 ) " +
           "( var sum 0 ) " +
           "( while ( < n 5 ) " +
             "( begin " +
               "( set sum ( + sum n ) ) " +
               "( set n ( + n 1 ) ) " +
               "( print n ) " +
               "( print sum ) ) ) " +
           "sum )",

        "( prog " +
          "( var y 3 ) " +
          "( fun f ( lambda x ( + x y ) ) ) " +
          "( fun f ( lambda u ( * u y ) ) ) " +
          "( fun h ( lambda v ( && v u ) ) ) " +
          "( apply  k f ) )", // function parameter

        "( prog ( var x ( apply ( fun f ( lambda x ( * x x ) ) ) x ) ) )",

        "( prog ( var y ( apply ( fun g ( lambda x ( * x y ) ) ) 0 ) ) )",

        "( prog ( var y 10 ) ( var y ( apply ( fun h ( lambda x ( * x y ) ) ) u ) ) )",

        "( prog " +
           "( fun mkAdd ( lambda a ( begin ( fun r ( lambda b ( + a b c ) ) ) r ) ) ) " +
           "( var add2 ( apply mkAdd 2 ) ) " +  // function return value
           "( apply add2 3 ) )",

        "( prog " +
           "( var c 1 ) " +
           "( fun mkAdd ( lambda a ( begin ( fun r ( lambda b ( + a b c ) ) ) r ) ) ) " +
           "( var add2 ( apply mkAdd 2 ) ) " +  // function return value
           "( apply add2 3 ) )",

         "( prog " +
           "( var x 20 ) " +
           "( begin " +
             "( var x 10 ) " +
             "( apply ( fun w ( lambda z ( + z x ) ) ) 5 ) ) )"
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         // Build the abstract syntax tree that represents the expression.
         try
         {
            Tree ast = Parser.parse( programs[i] );

            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );
            System.out.println( AST2Infix.ast2infix( ast ) + "\n" );

            StaticAnalysis.analyze( ast );

            // Create dot and png files from the AST.
            if(true)
            try
            {
               // Create the (empty) dot file.
               String baseName = String.format("Tests.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot_ver1.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main

}//Tests